<?php

namespace App\Util;

use App\Exception;

use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception as MailerException;

use App\Util\Security as Security;

class Util {
    
    private $connection;
    private $dbSchema = 'pt-br';
    
    public function __construct (\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
        //$this->loadDbSchema();
    }
    
    private function getDbSchema() {
        return $this->dbSchema;
    }
    
    private function loadDbSchema() {
        $configFile = file_get_contents(__DIR__ . "/../../environment/config.json");
        $config     = json_decode($configFile, true);
        
        if(isset($config["dbSchema"])){
            $this->dbSchema = $config["dbSchema"];
        }
    }
    
    public function removeDecimal($value, $offet = 2){
        return floatval(bcdiv($value,1,$offet));
    }
    
    public function subtract($a, $b){
        return floatval(bcsub($a,$b, 2));
    }
    
    public function add($a, $b){
        return floatval(bcadd($a,$b, 2));
    }
    
    public function mult($a, $b){
        return floatval(bcmul($a,$b, 2));
    }
    
    public function div($a, $b){
        return floatval(bcdiv($a,$b, 2));
    }
    
    /**
     *
     *
     * @param StoredProcedure $procedure The procedure to be executed.
     * @param array $paramValues Param values to override values present in Param instance.
     *
     * @return array The output parameters indexed by name.
     */
    public function executeProcedure(StoredProcedure $procedure, array $paramValues = array()) {
        $queryStmt = $this->getQueryString($procedure);
        $stmt = $this->connection->prepare($queryStmt);
        $valuesOutput = array();
        foreach ($procedure->getParams() as $param) {
            if ($param->isOutput()) {
                $valuesOutput[$param->getName()] = $param->getValue();
                $stmt->bindParam($param->getParamAlias(), $valuesOutput[$param->getName()], $param->getType(), $param->getLength());
            } else {
                $stmt->bindValue($param->getParamAlias(), $param->getValue(), $param->getType());
            }
        }

        $stmt->execute();
        return $valuesOutput;
    }

    /**
     * @param StoredProcedure $procedure
     * @return string
     */
    protected function getQueryString(StoredProcedure $procedure) {
        $paramNames = array_map(function (Param $param) {
            return ":" . $param->getName();
        }, $procedure->getParams());
        $paramString = implode(", ", $paramNames);
        return "CALL " . $procedure->getName() . " ( " . $paramString . " )";
    }

    public function writeLog($type, $fileName, $content) {
        global $CLIENT_DATA;
        $config = $CLIENT_DATA;

        $clientFolder = $this->connection->getParams();
        $clientFolder = (isset($clientFolder['dbname'])) ? $clientFolder['dbname'] : "undefined";
        $basePath     = $config['logPath'] . "/$clientFolder";
        

        if($type == 'network') {
            $folder      = $basePath . "/networkError/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";

            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, $content);
        }

    }
    
    private function formatNetworkError($networkData) {
        $eventTime = $networkData['eventTime'];
        $header    = implode($networkData['header'], "\r\n");
        $url       = $networkData['url'];
        $method    = $networkData['method'];
        $params    = is_array($networkData['params']) ? implode($networkData['params'], "\r\n") : $networkData['params'];
        $httpCode  = $networkData['httpCode'];
        $error     = $networkData['error'];
        $response  = $networkData['response'];

        $template = file_get_contents(__DIR__ . './../templates/networkError.txt');
        $templateParams = [
            ':EVENT'     => $eventTime,
            ':HEADER'    => $header,
            ':URL'       => $url,
            ':METHOD'    => $method,
            ':PARAMS'    => $params,
            ':HTTP_CODE' => $httpCode,
            ':ERROR'     => $error,
            ':RESPONSE'  => $response            
        ];

        return str_replace(array_keys($templateParams), array_values($templateParams), $template);
    }   
    
    public function tofloat($num) {
        $num = str_replace(",", ".", $num);
        return floatval($num);

    }
    
    
    
    public function getCurrentTimezone($state) {
        $timezones = [
            "AC" => "America/Rio_branco",   "AL" => "America/Maceio",
            "AP" => "America/Belem",        "AM" => "America/Manaus",
            "BA" => "America/Bahia",        "CE" => "America/Fortaleza",
            "DF" => "America/Fortaleza",    "ES" => "America/Fortaleza",
            "GO" => "America/Fortaleza",    "MA" => "America/Fortaleza",
            "MT" => "America/Cuiaba",       "MS" => "America/Campo_Grande",
            "MG" => "America/Fortaleza",    "PR" => "America/Fortaleza",
            "PB" => "America/Fortaleza",    "PA" => "America/Belem",
            "PE" => "America/Recife",       "PI" => "America/Fortaleza",
            "RJ" => "America/Fortaleza",    "RN" => "America/Fortaleza",
            "RS" => "America/Fortaleza",    "RO" => "America/Porto_Velho",
            "RR" => "America/Boa_Vista",    "SC" => "America/Fortaleza",
            "SE" => "America/Maceio",       "SP" => "America/Fortaleza",
            "TO" => "America/Araguaia",    
        ];
       
        return $timezones[$state];
    }
    
    public function getCurrentDateTime() {
    	return (new \DateTime())->format('Y-m-d\TH:i:s:uO');
    }    
    
    
    public function removeAcentos($texto) {
        return preg_replace(array("/(á|à|ã|â|ä)/","/(Á|À|Ã|Â|Ä)/","/(é|è|ê|ë)/","/(É|È|Ê|Ë)/","/(í|ì|î|ï)/","/(Í|Ì|Î|Ï)/","/(ó|ò|õ|ô|ö)/","/(Ó|Ò|Õ|Ô|Ö)/","/(ú|ù|û|ü)/","/(Ú|Ù|Û|Ü)/","/(ñ)/","/(Ñ)/","/(ç)/","/(Ç)/"),explode(" ","a A e E i I o O u U n N c C"),$texto);
    }
    
}
